@@ -70,10 +70,15 @@ object Constants {
     const val yes = "Yes"
     const val no = "No"
     const val loading = "Loading..."
+    const val working = "Working..."
 
     const val barbarians = "Barbarians"
     const val spectator = "Spectator"
 
+    const val embarked = "Embarked"
+    const val wounded = "Wounded"
+
+
     const val rising = "Rising"
     const val lowering = "Lowering"
     const val remove = "Remove "
