@@ -107,21 +107,6 @@ class WorkerAutomation(
     ///////////////////////////////////////// Helpers /////////////////////////////////////////
 
     companion object {
-        /** Maps to instance [WorkerAutomation.automateWorkerAction] knowing only the MapUnit */
-        fun automateWorkerAction(unit: MapUnit, tilesWhereWeWillBeCaptured: Set<Tile>) {
-            unit.civ.getWorkerAutomation().automateWorkerAction(unit, tilesWhereWeWillBeCaptured)
-        }
-
-        /** Convenience shortcut supports old calling syntax for [WorkerAutomation.getPriority] */
-        fun getPriority(tile: Tile, civInfo: Civilization): Int {
-            return civInfo.getWorkerAutomation().getPriority(tile)
-        }
-
-        /** Convenience shortcut supports old calling syntax for [WorkerAutomation.evaluateFortPlacement] */
-        fun evaluateFortPlacement(tile: Tile, civInfo: Civilization, isCitadel: Boolean): Boolean {
-            return civInfo.getWorkerAutomation().evaluateFortPlacement(tile, isCitadel)
-        }
-
         /** For console logging only */
         private fun MapUnit.label() = toString() + " " + getTile().position.toString()
     }
@@ -135,7 +120,7 @@ class WorkerAutomation(
         val currentTile = unit.getTile()
         val tileToWork = findTileToWork(unit, tilesWhereWeWillBeCaptured)
 
-        if (getPriority(tileToWork, civInfo) < 3) { // building roads is more important
+        if (civInfo.getWorkerAutomation().getPriority(tileToWork) < 3) { // building roads is more important
             if (tryConnectingCities(unit)) return
         }
 
@@ -285,7 +270,7 @@ class WorkerAutomation(
                 .filter {
                     it !in tilesToAvoid
                     && (it.civilianUnit == null || it == currentTile)
-                    && (it.owningCity == null || it.getOwner()==civInfo)
+                    && (it.owningCity == null || it.getOwner() == civInfo)
                     && getPriority(it) > 1
                     && it.getTilesInDistance(2)  // don't work in range of enemy cities
                         .none { tile -> tile.isCityCenter() && tile.getCity()!!.civ.isAtWarWith(civInfo) }
@@ -294,13 +279,16 @@ class WorkerAutomation(
                 }
                 .sortedByDescending { getPriority(it) }
 
+        // Carthage can move through mountains, special case
+        // If there is a non-damage dealing tile available, move to that tile, otherwise move to the damage dealing tile
         // These are the expensive calculations (tileCanBeImproved, canReach), so we only apply these filters after everything else it done.
-        val selectedTile = workableTiles.firstOrNull { unit.movement.canReach(it) && (tileCanBeImproved(unit, it) || it.isPillaged()) }
+        val selectedTile =
+            workableTiles.sortedByDescending { tile -> unit.getDamageFromTerrain(tile) <= 0 }.firstOrNull { unit.movement.canReach(it) && (tileCanBeImproved(unit, it) || it.isPillaged()) }
+                ?: return currentTile
 
-        return if (selectedTile != null
-                && ((!tileCanBeImproved(unit, currentTile) && !currentTile.isPillaged()) // current tile is unimprovable
+        return if ((!tileCanBeImproved(unit, currentTile) && !currentTile.isPillaged()) // current tile is unimprovable
                     || !workableTiles.contains(currentTile) // current tile is unworkable by city
-                    || getPriority(selectedTile) > getPriority(currentTile)))  // current tile is less important
+                    || getPriority(selectedTile) > getPriority(currentTile))  // current tile is less important
             selectedTile
         else currentTile
     }
@@ -344,7 +332,7 @@ class WorkerAutomation(
     /**
      * Calculate a priority for improving a tile
      */
-    private fun getPriority(tile: Tile): Int {
+    fun getPriority(tile: Tile): Int {
         var priority = 0
         if (tile.getOwner() == civInfo) {
             priority += 2
@@ -389,16 +377,15 @@ class WorkerAutomation(
 
         val lastTerrain = tile.lastTerrain
 
-        fun isUnbuildableAndRemovable(terrain: Terrain): Boolean = terrain.unbuildable
-                && ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
-
+        fun isRemovable(terrain: Terrain): Boolean = ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
 
         val improvementStringForResource: String? = when {
             tile.resource == null || !tile.hasViewableResource(civInfo) -> null
             tile.terrainFeatures.isNotEmpty()
-                    && isUnbuildableAndRemovable(lastTerrain)
-                    && !tile.providesResources(civInfo)
-                    && !isResourceImprovementAllowedOnFeature(tile, potentialTileImprovements) -> Constants.remove + lastTerrain.name
+                && lastTerrain.unbuildable
+                && isRemovable(lastTerrain)
+                && !tile.providesResources(civInfo)
+                && !isResourceImprovementAllowedOnFeature(tile, potentialTileImprovements) -> Constants.remove + lastTerrain.name
             else -> tile.tileResource.getImprovements().filter { it in potentialTileImprovements || it==tile.improvement }
                 .maxByOrNull { getRankingWithImprovement(it) }
         }
@@ -411,12 +398,14 @@ class WorkerAutomation(
             tile.resource != null && tile.tileResource.getImprovements().any() -> return null
             bestBuildableImprovement == null -> null
 
-            tile.improvement!=null && getRankingWithImprovement(tile.improvement!!) > getRankingWithImprovement(bestBuildableImprovement.name)
+            tile.improvement != null &&
+                    getRankingWithImprovement(tile.improvement!!) > getRankingWithImprovement(bestBuildableImprovement.name)
                 -> null // What we have is better, even if it's pillaged we should repair it
 
             lastTerrain.let {
-                isUnbuildableAndRemovable(it) &&
-                        (Automation.rankStatsValue(it, civInfo) < 0 || it.hasUnique(UniqueType.NullifyYields) )
+                isRemovable(it) &&
+                    (Automation.rankStatsValue(it, civInfo) < 0
+                        || it.hasUnique(UniqueType.NullifyYields))
             } -> Constants.remove + lastTerrain.name
 
             else -> bestBuildableImprovement.name
@@ -463,7 +452,7 @@ class WorkerAutomation(
      * @param  isCitadel Controls within borders check - true also allows 1 tile outside borders
      * @return Yes please build a Fort here
      */
-    private fun evaluateFortPlacement(tile: Tile, isCitadel: Boolean): Boolean {
+    fun evaluateFortPlacement(tile: Tile, isCitadel: Boolean): Boolean {
         //todo Is the Citadel code dead anyway? If not - why does the nearestTiles check not respect the param?
 
         // build on our land only
