@@ -12,12 +12,14 @@ import com.unciv.logic.trade.Trade
 import com.unciv.logic.trade.TradeOffer
 import com.unciv.logic.trade.TradeType
 import com.unciv.models.ruleset.tile.ResourceSupplyList
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.extensions.toPercent
 import kotlin.math.ceil
 import kotlin.math.max
 import kotlin.math.min
+import kotlin.math.sign
 
 enum class RelationshipLevel(val color: Color) {
     // War is tested separately for the Diplomacy Screen. Colored RED.
@@ -148,9 +150,9 @@ class DiplomacyManager() : IsPartOfGameInfoSerialization {
         return toReturn
     }
 
-    constructor(civilization: Civilization, OtherCivName: String) : this() {
+    constructor(civilization: Civilization, mOtherCivName: String) : this() {
         civInfo = civilization
-        otherCivName = OtherCivName
+        otherCivName = mOtherCivName
         updateHasOpenBorders()
     }
 
@@ -185,7 +187,7 @@ class DiplomacyManager() : IsPartOfGameInfoSerialization {
      */
     private fun compareRelationshipLevel(level: RelationshipLevel, comparesAs: Int): Boolean {
         if (!civInfo.isCityState())
-            return relationshipLevel().compareTo(level) == comparesAs
+            return relationshipLevel().compareTo(level).sign == comparesAs
         return when(level) {
             RelationshipLevel.Afraid -> when {
                 comparesAs < 0 -> getInfluence() < 0
@@ -199,7 +201,9 @@ class DiplomacyManager() : IsPartOfGameInfoSerialization {
             }
             else ->
                 // Outside the potentially expensive questions, do it the easy way
-                relationshipLevel().compareTo(level) == comparesAs
+                // Except - Enum.compareTo does not behave quite like other compareTo's
+                // or like kotlinlang.org says, thus the `sign`
+                relationshipLevel().compareTo(level).sign == comparesAs
         }
     }
     /** @see compareRelationshipLevel */
@@ -404,6 +408,7 @@ class DiplomacyManager() : IsPartOfGameInfoSerialization {
         val isResourceFilter: (TradeOffer) -> Boolean = {
             (it.type == TradeType.Strategic_Resource || it.type == TradeType.Luxury_Resource)
                     && resourcesMap.containsKey(it.name)
+                    && !resourcesMap[it.name]!!.isStockpiled()
         }
         for (trade in trades) {
             for (offer in trade.ourOffers.filter(isResourceFilter))
@@ -436,8 +441,8 @@ class DiplomacyManager() : IsPartOfGameInfoSerialization {
 
             // Every cancelled trade can change this - if 1 resource is missing,
             // don't cancel all trades of that resource, only cancel one (the first one, as it happens, since they're added chronologically)
-            val negativeCivResources = civInfo.getCivResources()
-                .filter { it.amount < 0 }.map { it.resource.name }
+            val negativeCivResources = civInfo.getCivResourceSupply()
+                .filter { it.amount < 0 && !it.resource.isStockpiled() }.map { it.resource.name }
 
             for (offer in trade.ourOffers) {
                 if (offer.type in listOf(TradeType.Luxury_Resource, TradeType.Strategic_Resource)
@@ -901,9 +906,11 @@ class DiplomacyManager() : IsPartOfGameInfoSerialization {
             thirdCiv.getDiplomacyManager(civInfo).setFriendshipBasedModifier()
         }
 
-        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponDeclaringFriendship))
+        // Ignore contitionals as triggerCivwideUnique will check again, and that would break
+        // UniqueType.ConditionalChance - 25% declared chance would work as 6% actual chance
+        for (unique in civInfo.getTriggeredUniques(UniqueType.TriggerUponDeclaringFriendship, StateForConditionals.IgnoreConditionals))
             UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo)
-        for (unique in otherCiv().getTriggeredUniques(UniqueType.TriggerUponDeclaringFriendship))
+        for (unique in otherCiv().getTriggeredUniques(UniqueType.TriggerUponDeclaringFriendship, StateForConditionals.IgnoreConditionals))
             UniqueTriggerActivation.triggerCivwideUnique(unique, otherCiv())
     }
 
