@@ -196,10 +196,10 @@ class TechManager : IsPartOfGameInfoSerialization {
     private fun addCurrentScienceToScienceOfLast8Turns() {
         // The Science the Great Scientist generates does not include Science from Policies, Trade routes and City-States.
         var allCitiesScience = 0f
-        civInfo.cities.forEach { it ->
+        civInfo.cities.forEach {
             val totalBaseScience = it.cityStats.baseStatTree.totalStats.science
             val totalBonusPercents = it.cityStats.statPercentBonusTree.children.asSequence()
-                .filter { it.key != "Policies" }.map { it.value.totalStats.science }.sum()
+                .filter { it2 -> it2.key != "Policies" }.map { it2 ->  it2.value.totalStats.science }.sum()
             allCitiesScience += totalBaseScience * totalBonusPercents.toPercent()
         }
         scienceOfLast8Turns[civInfo.gameInfo.turns % 8] = allCitiesScience.toInt()
@@ -276,7 +276,7 @@ class TechManager : IsPartOfGameInfoSerialization {
         addTechnology(techName)
     }
 
-    fun addTechnology(techName: String) {
+    fun addTechnology(techName: String, showNotification: Boolean = true) {
         val isNewTech = techsResearched.add(techName)
 
         // this is to avoid concurrent modification problems
@@ -302,21 +302,26 @@ class TechManager : IsPartOfGameInfoSerialization {
         updateTransientBooleans()
         for (city in civInfo.cities) {
             city.cityStats.update()
-            city.updateCitizens = true
+            city.reassignPopulationDeferred()   
         }
 
-        if (!civInfo.isSpectator())
+        if (!civInfo.isSpectator() && showNotification)
             civInfo.addNotification("Research of [$techName] has completed!", TechAction(techName),
                 NotificationCategory.General,
                 NotificationIcon.Science)
         if (isNewTech)
             civInfo.popupAlerts.add(PopupAlert(AlertType.TechResearched, techName))
 
+        val revealedResources = getRuleset().tileResources.values.filter { techName == it.revealedBy }
         if (civInfo.playerType == PlayerType.Human) {
-            for (revealedResource in getRuleset().tileResources.values.filter { techName == it.revealedBy }) {
-                civInfo.gameInfo.notifyExploredResources(civInfo, revealedResource.name, 5, false)
+            for (revealedResource in revealedResources) {
+                civInfo.gameInfo.notifyExploredResources(civInfo, revealedResource.name, 5)
             }
         }
+        // In the case of a player hurrying research, this civ's resource availability may now be out of date
+        // - e.g. when an owned tile by luck already has an appropriate improvement or when a tech provides a resource.
+        // That can be seen on WorldScreenTopBar, so better update.
+        civInfo.cache.updateCivResources()
 
         obsoleteOldUnits(techName)
 
@@ -330,7 +335,7 @@ class TechManager : IsPartOfGameInfoSerialization {
                 MayaLongCountAction(), NotificationCategory.General, MayaCalendar.notificationIcon)
         }
 
-        moveToNewEra()
+        moveToNewEra(showNotification)
         updateResearchProgress()
     }
 
@@ -371,7 +376,7 @@ class TechManager : IsPartOfGameInfoSerialization {
         // Add notifications for obsolete units/constructions
         for ((unit, cities) in unitUpgrades) {
             if (cities.isEmpty()) continue
-            val locationAction = LocationAction(cities.mapTo(ArrayList(cities.size)) { it.location })
+            val locationAction = LocationAction(cities.asSequence().map { it.location })
             val cityText = if (cities.size == 1) "[${cities.first().name}]"
                 else "[${cities.size}] cities"
             val newUnit = obsoleteUnits[unit]?.name
@@ -385,34 +390,36 @@ class TechManager : IsPartOfGameInfoSerialization {
         }
     }
 
-    private fun moveToNewEra() {
+    private fun moveToNewEra(showNotification: Boolean = true) {
         val previousEra = civInfo.getEra()
         updateEra()
         val currentEra = civInfo.getEra()
         if (previousEra != currentEra) {
-            if(!civInfo.isSpectator())
-                civInfo.addNotification(
-                    "You have entered the [$currentEra]!",
-                    NotificationCategory.General,
-                    NotificationIcon.Science
-                )
-            if (civInfo.isMajorCiv()) {
-                for (knownCiv in civInfo.getKnownCivs()) {
-                    knownCiv.addNotification(
-                        "[${civInfo.civName}] has entered the [$currentEra]!",
-                        NotificationCategory.General, civInfo.civName, NotificationIcon.Science
-                    )
-                }
-            }
-            for (policyBranch in getRuleset().policyBranches.values.filter {
-                it.era == currentEra.name && civInfo.policies.isAdoptable(it)
-            }) {
-                if (!civInfo.isSpectator())
+            if(showNotification) {
+                if(!civInfo.isSpectator())
                     civInfo.addNotification(
-                        "[${policyBranch.name}] policy branch unlocked!",
+                        "You have entered the [$currentEra]!",
                         NotificationCategory.General,
-                        NotificationIcon.Culture
+                        NotificationIcon.Science
                     )
+                if (civInfo.isMajorCiv()) {
+                    for (knownCiv in civInfo.getKnownCivs()) {
+                        knownCiv.addNotification(
+                            "[${civInfo.civName}] has entered the [$currentEra]!",
+                            NotificationCategory.General, civInfo.civName, NotificationIcon.Science
+                        )
+                    }
+                }
+                for (policyBranch in getRuleset().policyBranches.values.filter {
+                    it.era == currentEra.name && civInfo.policies.isAdoptable(it)
+                }) {
+                    if (!civInfo.isSpectator())
+                        civInfo.addNotification(
+                            "[${policyBranch.name}] policy branch unlocked!",
+                            NotificationCategory.General,
+                            NotificationIcon.Culture
+                        )
+                }
             }
 
             val erasPassed = getRuleset().eras.values
