@@ -378,6 +378,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
 
     fun isEmbarked(): Boolean {
         if (!baseUnit.isLandUnit()) return false
+        if (cache.canMoveOnWater) return false
         return currentTile.isWater
     }
 
@@ -817,7 +818,7 @@ class MapUnit : IsPartOfGameInfoSerialization {
     fun canBuildImprovement(improvement: TileImprovement, tile: Tile = currentTile): Boolean {
         // Workers (and similar) should never be able to (instantly) construct things, only build them
         // HOWEVER, they should be able to repair such things if they are pillaged
-        if (improvement.turnsToBuild == 0
+        if (improvement.turnsToBuild == -1
             && improvement.name != Constants.cancelImprovementOrder
             && tile.improvementInProgress != improvement.name
         ) return false
