@@ -91,6 +91,7 @@ class Ruleset {
     val quests = LinkedHashMap<String, Quest>()
     val specialists = LinkedHashMap<String, Specialist>()
     val technologies = LinkedHashMap<String, Technology>()
+    val techColumns = ArrayList<TechColumn>()
     val terrains = LinkedHashMap<String, Terrain>()
     val tileImprovements = LinkedHashMap<String, TileImprovement>()
     val tileResources = LinkedHashMap<String, TileResource>()
@@ -152,6 +153,7 @@ class Ruleset {
                 technologies.remove(it)
             }
         technologies.putAll(ruleset.technologies)
+        techColumns.addAll(ruleset.techColumns)
         terrains.putAll(ruleset.terrains)
         tileImprovements.putAll(ruleset.tileImprovements)
         tileResources.putAll(ruleset.tileResources)
@@ -160,7 +162,7 @@ class Ruleset {
         cityStateTypes.putAll(ruleset.cityStateTypes)
         ruleset.modOptions.unitsToRemove
             .flatMap { unitToRemove ->
-                units.filter { it.apply { value.ruleset=this@Ruleset }.value.matchesFilter(unitToRemove) }.keys
+                units.filter { it.apply { value.ruleset = this@Ruleset }.value.matchesFilter(unitToRemove) }.keys
             }.toSet().forEach {
                 units.remove(it)
             }
@@ -168,30 +170,7 @@ class Ruleset {
         modOptions.uniques.addAll(ruleset.modOptions.uniques)
         modOptions.constants.merge(ruleset.modOptions.constants)
 
-        // Allow each mod to define their own columns, and if there's a conflict, later mods will be shifted right
-        // We should never be editing the original ruleset objects, only copies
-        val addRulesetUnitPromotionClones = ruleset.unitPromotions.values.map { it.clone() }
-        val existingPromotionLocations =
-                unitPromotions.values.map { "${it.row}/${it.column}" }.toHashSet()
-        val promotionsWithConflictingLocations = addRulesetUnitPromotionClones.filter {
-            existingPromotionLocations.contains("${it.row}/${it.column}")
-        }
-        val columnsWithConflictingLocations =
-                promotionsWithConflictingLocations.map { it.column }.distinct()
-
-        if (columnsWithConflictingLocations.isNotEmpty()) {
-            var highestExistingColumn = unitPromotions.values.maxOf { it.column }
-            for (conflictingColumn in columnsWithConflictingLocations) {
-                highestExistingColumn += 1
-                val newColumn = highestExistingColumn
-                for (promotion in addRulesetUnitPromotionClones)
-                    if (promotion.column == conflictingColumn)
-                        promotion.column = newColumn
-            }
-        }
-        val finalModUnitPromotionsMap = addRulesetUnitPromotionClones.associateBy { it.name }
-
-        unitPromotions.putAll(finalModUnitPromotionsMap)
+        unitPromotions.putAll(ruleset.unitPromotions)
 
         mods += ruleset.mods
     }
@@ -212,6 +191,7 @@ class Ruleset {
         ruinRewards.clear()
         specialists.clear()
         technologies.clear()
+        techColumns.clear()
         terrains.clear()
         tileImprovements.clear()
         tileResources.clear()
@@ -264,6 +244,7 @@ class Ruleset {
         if (techFile.exists()) {
             val techColumns = json().fromJsonFile(Array<TechColumn>::class.java, techFile)
             for (techColumn in techColumns) {
+                this.techColumns.add(techColumn)
                 for (tech in techColumn.techs) {
                     if (tech.cost == 0) tech.cost = techColumn.techCost
                     tech.column = techColumn
@@ -308,14 +289,6 @@ class Ruleset {
         val promotionsFile = folderHandle.child("UnitPromotions.json")
         if (promotionsFile.exists()) unitPromotions += createHashmap(json().fromJsonFile(Array<Promotion>::class.java, promotionsFile))
 
-        var topRow = unitPromotions.values.filter { it.column == 0 }.maxOfOrNull { it.row } ?: -1
-        for (promotion in unitPromotions.values)
-            if (promotion.row == -1){
-                promotion.column = 0
-                topRow += 1
-                promotion.row = topRow
-            }
-
         val questsFile = folderHandle.child("Quests.json")
         if (questsFile.exists()) quests += createHashmap(json().fromJsonFile(Array<Quest>::class.java, questsFile))
 
@@ -456,10 +429,11 @@ class Ruleset {
      *  */
     fun updateBuildingCosts() {
         for (building in buildings.values) {
-            if (building.cost == 0 && building.getMatchingUniques(UniqueType.Unbuildable).none { it.conditionals.isEmpty() }) {
+            if (building.cost == -1 && building.getMatchingUniques(UniqueType.Unbuildable).none { it.conditionals.isEmpty() }) {
                 val column = technologies[building.requiredTech]?.column
-                        ?: throw UncivShowableException("Building '[${building.name}]' is buildable and therefore must either have an explicit cost or reference an existing tech.")
-                building.cost = if (building.isAnyWonder()) column.wonderCost else column.buildingCost
+                if (column != null) {
+                    building.cost = if (building.isAnyWonder()) column.wonderCost else column.buildingCost
+                }
             }
         }
     }
