@@ -31,6 +31,7 @@ class RulesetValidator(val ruleset: Ruleset) {
 
         val lines = RulesetErrorList()
 
+        /********************** Ruleset Invariant Part **********************/
         // Checks for all mods - only those that can succeed without loading a base ruleset
         // When not checking the entire ruleset, we can only really detect ruleset-invariant errors in uniques
 
@@ -50,17 +51,27 @@ class RulesetValidator(val ruleset: Ruleset) {
 
         for (tech in ruleset.technologies.values) {
             for (otherTech in ruleset.technologies.values) {
-                if (tech != otherTech && otherTech.column == tech.column && otherTech.row == tech.row)
-                    lines += "${tech.name} is in the same row as ${otherTech.name}!"
+                if (tech != otherTech && otherTech.column?.columnNumber == tech.column?.columnNumber && otherTech.row == tech.row)
+                    lines += "${tech.name} is in the same row and column as ${otherTech.name}!"
             }
 
             checkUniques(tech, lines, rulesetInvariant, tryFixUnknownUniques)
         }
 
+        for (techColumn in ruleset.techColumns){
+            if (techColumn.columnNumber < 0)
+                lines+= "Tech Column number ${techColumn.columnNumber} is negative"
+            if (techColumn.buildingCost == -1)
+                lines.add("Tech Column number ${techColumn.columnNumber} has no explicit building cost", RulesetErrorSeverity.Warning)
+            if (techColumn.wonderCost == -1)
+                lines.add("Tech Column number ${techColumn.columnNumber} has no explicit wonder cost", RulesetErrorSeverity.Warning)
+        }
+
         for (building in ruleset.buildings.values) {
-            if (building.requiredTech == null && building.cost == 0 && !building.hasUnique(
+            if (building.requiredTech == null && building.cost == -1 && !building.hasUnique(
                         UniqueType.Unbuildable))
-                lines += "${building.name} is buildable and therefore must either have an explicit cost or reference an existing tech!"
+                lines.add("${building.name} is buildable and therefore should either have an explicit cost or reference an existing tech!",
+                    RulesetErrorSeverity.Warning)
 
             checkUniques(building, lines, rulesetInvariant, tryFixUnknownUniques)
 
@@ -128,6 +139,8 @@ class RulesetValidator(val ruleset: Ruleset) {
         // Quit here when no base ruleset is loaded - references cannot be checked
         if (!ruleset.modOptions.isBaseRuleset) return lines
 
+        /********************** Ruleset Specific Part **********************/
+
         val vanillaRuleset = RulesetCache.getVanillaRuleset()  // for UnitTypes fallback
 
 
@@ -166,8 +179,7 @@ class RulesetValidator(val ruleset: Ruleset) {
                     lines += "${unit.name} contains promotion $promotion which does not exist!"
             if (!ruleset.unitTypes.containsKey(unit.unitType) && (ruleset.unitTypes.isNotEmpty() || !vanillaRuleset.unitTypes.containsKey(unit.unitType)))
                 lines += "${unit.name} is of type ${unit.unitType}, which does not exist!"
-            for (unique in unit.getMatchingUniques(UniqueType.ConstructImprovementConsumingUnit)
-                + unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)) {
+            for (unique in unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)) {
                 val improvementName = unique.params[0]
                 if (ruleset.tileImprovements[improvementName]==null) continue // this will be caught in the checkUniques
                 if ((ruleset.tileImprovements[improvementName] as Stats).none() &&
@@ -485,12 +497,11 @@ class RulesetValidator(val ruleset: Ruleset) {
 
         val typeComplianceErrors = unique.type.getComplianceErrors(unique, ruleset)
         for (complianceError in typeComplianceErrors) {
-            // TODO: Make this Error eventually, this is Not Good
             if (complianceError.errorSeverity <= severityToReport)
                 rulesetErrors.add(RulesetError("$name's unique \"${unique.text}\" contains parameter ${complianceError.parameterName}," +
                         " which does not fit parameter type" +
                         " ${complianceError.acceptableParameterTypes.joinToString(" or ") { it.parameterName }} !",
-                    RulesetErrorSeverity.Warning
+                    complianceError.errorSeverity.getRulesetErrorSeverity(severityToReport)
                 ))
         }
 
@@ -506,9 +517,11 @@ class RulesetValidator(val ruleset: Ruleset) {
                         conditional.type.getComplianceErrors(conditional, ruleset)
                 for (complianceError in conditionalComplianceErrors) {
                     if (complianceError.errorSeverity == severityToReport)
-                        rulesetErrors += "$name's unique \"${unique.text}\" contains the conditional \"${conditional.text}\"." +
+                        rulesetErrors.add(RulesetError( "$name's unique \"${unique.text}\" contains the conditional \"${conditional.text}\"." +
                                 " This contains the parameter ${complianceError.parameterName} which does not fit parameter type" +
-                                " ${complianceError.acceptableParameterTypes.joinToString(" or ") { it.parameterName }} !"
+                                " ${complianceError.acceptableParameterTypes.joinToString(" or ") { it.parameterName }} !",
+                            complianceError.errorSeverity.getRulesetErrorSeverity(severityToReport)
+                        ))
                 }
             }
         }
@@ -539,6 +552,7 @@ class RulesetValidator(val ruleset: Ruleset) {
 
 
 class RulesetError(val text:String, val errorSeverityToReport: RulesetErrorSeverity)
+
 enum class RulesetErrorSeverity(val color: Color) {
     OK(Color.GREEN),
     WarningOptionsOnly(Color.YELLOW),
@@ -555,6 +569,23 @@ class RulesetErrorList : ArrayList<RulesetError>() {
         add(RulesetError(text, errorSeverityToReport))
     }
 
+    override fun add(element: RulesetError): Boolean {
+        // Suppress duplicates due to the double run of some checks for invariant/specific,
+        // Without changing collection type or making RulesetError obey the equality contract
+        val existing = firstOrNull { it.text == element.text }
+            ?: return super.add(element)
+        if (existing.errorSeverityToReport >= element.errorSeverityToReport) return false
+        remove(existing)
+        return super.add(element)
+    }
+
+    override fun addAll(elements: Collection<RulesetError>): Boolean {
+        var result = false
+        for (element in elements)
+            if (add(element)) result = true
+        return result
+    }
+
     fun getFinalSeverity(): RulesetErrorSeverity {
         if (isEmpty()) return RulesetErrorSeverity.OK
         return this.maxOf { it.errorSeverityToReport }
@@ -572,5 +603,10 @@ class RulesetErrorList : ArrayList<RulesetError>() {
     fun getErrorText(filter: (RulesetError)->Boolean) =
             filter(filter)
                 .sortedByDescending { it.errorSeverityToReport }
-                .joinToString("\n") { it.errorSeverityToReport.name + ": " + it.text }
+                .joinToString("\n") {
+                    it.errorSeverityToReport.name + ": " +
+                        // This will go through tr(), unavoidably, which will move the conditionals
+                        // out of place. Prevent via kludge:
+                        it.text.replace('<','〈').replace('>','〉')
+                }
 }
