@@ -27,7 +27,7 @@ import kotlin.math.pow
  in contrast to MapUnit, which is a specific unit of a certain type that appears on the map */
 class BaseUnit : RulesetObject(), INonPerpetualConstruction {
 
-    override var cost: Int = 0
+    override var cost: Int = -1
     override var hurryCostModifier: Int = 0
     var movement: Int = 0
     var strength: Int = 0
@@ -301,15 +301,17 @@ class BaseUnit : RulesetObject(), INonPerpetualConstruction {
             "Nuclear Weapon" -> isNuclearWeapon()
             "Great Person" -> isGreatPerson()
             "Religious" -> hasUnique(UniqueType.ReligiousUnit)
+
             else -> {
                 if (type.matchesFilter(filter)) return true
+                if (requiredTech != null && ruleset.technologies[requiredTech]?.matchesFilter(filter)==true) return true
                 if (
                     // Uniques using these kinds of filters should be deprecated and replaced with adjective-only parameters
                     filter.endsWith(" units")
                     // "military units" --> "Military", using invariant locale
                     && matchesFilter(filter.removeSuffix(" units").lowercase().replaceFirstChar { it.uppercaseChar() })
                 ) return true
-                return uniques.contains(filter)
+                return uniqueMap.contains(filter)
             }
         }
     }
