@@ -14,7 +14,6 @@ import com.unciv.logic.civilization.PopupAlert
 import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
 import com.unciv.logic.civilization.diplomacy.DiplomacyManager
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
-import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers.*
 import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
 import com.unciv.logic.civilization.diplomacy.RelationshipLevel
 import com.unciv.logic.civilization.managers.AssignedQuest
@@ -31,13 +30,13 @@ import com.unciv.ui.audio.MusicMood
 import com.unciv.ui.audio.MusicTrackChooserFlags
 import com.unciv.ui.components.ColorMarkupLabel
 import com.unciv.ui.components.Fonts
-import com.unciv.ui.components.KeyCharAndCode
+import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.components.UncivTooltip.Companion.addTooltip
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.disable
-import com.unciv.ui.components.extensions.keyShortcuts
-import com.unciv.ui.components.extensions.onActivation
-import com.unciv.ui.components.extensions.onClick
+import com.unciv.ui.components.input.keyShortcuts
+import com.unciv.ui.components.input.onActivation
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.extensions.setFontSize
 import com.unciv.ui.components.extensions.surroundWithCircle
 import com.unciv.ui.components.extensions.toLabel
@@ -59,7 +58,6 @@ import com.unciv.ui.components.AutoScrollPane as ScrollPane
  * When [selectCiv] and [selectTrade] are supplied, that Trade for that Civilization is selected, used for the counter-offer option from `TradePopup`.
  * Note calling this with [selectCiv] a City State and [selectTrade] supplied is **not allowed**.
  */
-@Suppress("KDocUnresolvedReference")  // Mentioning non-field parameters is flagged, but they work anyway
 class DiplomacyScreen(
     private val viewingCiv: Civilization,
     private val selectCiv: Civilization? = null,
@@ -351,7 +349,8 @@ class DiplomacyScreen(
                 updateRightSide(otherCiv)
             }.open()
         }
-        if (isNotPlayersTurn() || !otherCiv.cityStateFunctions.otherCivCanWithdrawProtection(viewingCiv)) revokeProtectionButton.disable()
+        if (isNotPlayersTurn() || !otherCiv.cityStateFunctions.otherCivCanWithdrawProtection(viewingCiv))
+            revokeProtectionButton.disable()
         return revokeProtectionButton
     }
 
@@ -369,7 +368,8 @@ class DiplomacyScreen(
                 updateRightSide(otherCiv)
             }.open()
         }
-        if (isNotPlayersTurn() || !otherCiv.cityStateFunctions.otherCivCanPledgeProtection(viewingCiv)) protectionButton.disable()
+        if (isNotPlayersTurn() || !otherCiv.cityStateFunctions.otherCivCanPledgeProtection(viewingCiv))
+            protectionButton.disable()
         return protectionButton
     }
 
@@ -424,7 +424,7 @@ class DiplomacyScreen(
         if (otherCiv.cities.isEmpty()) return null
         val improvableResourceTiles = getImprovableResourceTiles(otherCiv)
         val improvements =
-            otherCiv.gameInfo.ruleset.tileImprovements.filter { it.value.turnsToBuild != 0 }
+            otherCiv.gameInfo.ruleset.tileImprovements.filter { it.value.turnsToBuild != -1 }
         var needsImprovements = false
 
         for (improvableTile in improvableResourceTiles)
@@ -462,7 +462,8 @@ class DiplomacyScreen(
             for (city in newCities)
                 viewingCiv.popupAlerts.add(PopupAlert(AlertType.DiplomaticMarriage, city.id))   // Player gets to choose between annex and puppet
         }
-        if (isNotPlayersTurn() || !otherCiv.cityStateFunctions.canBeMarriedBy(viewingCiv)) diplomaticMarriageButton.disable()
+        if (isNotPlayersTurn() || !otherCiv.cityStateFunctions.canBeMarriedBy(viewingCiv))
+            diplomaticMarriageButton.disable()
         return diplomaticMarriageButton
     }
 
@@ -825,8 +826,8 @@ class DiplomacyScreen(
         val diplomacyModifiersTable = Table()
         for (modifier in otherCivDiplomacyManager.diplomaticModifiers) {
             // Angry about attacked CS and destroyed CS do not stack
-            if (modifier.key == AttackedProtectedMinor.name
-                && otherCivDiplomacyManager.hasModifier(DestroyedProtectedMinor))
+            if (modifier.key == DiplomaticModifiers.AttackedProtectedMinor.name
+                && otherCivDiplomacyManager.hasModifier(DiplomaticModifiers.DestroyedProtectedMinor))
                 continue
 
             var text = DiplomaticModifiers.valueOf(modifier.key).text.tr() + " "
