@@ -7,16 +7,17 @@ import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.stats.Stats
 import com.unciv.testing.GdxTestRunner
-import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsPillage
 import org.junit.Assert
 import org.junit.Before
 import org.junit.Test
 import org.junit.runner.RunWith
+import kotlin.math.abs
 
 @RunWith(GdxTestRunner::class)
 class GlobalUniquesTests {
 
     private lateinit var game: TestGame
+    private val epsilon = 0.01 // for float comparisons
 
     @Before
     fun initTheWorld() {
@@ -524,6 +525,98 @@ class GlobalUniquesTests {
 
     //endregion
 
+    // region Other Global Uniques
+
+    // region growth
+
+    @Test
+    fun growthPercentBonusTest() {
+        val civInfo = game.addCiv()
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val cityInfo = game.addCity(civInfo, tile, true)
+        val building = game.createBuilding("[+100]% growth [in all cities]")
+        cityInfo.cityConstructions.addBuilding(building.name)
+
+        cityInfo.cityStats.update()
+        Assert.assertTrue(cityInfo.cityStats.finalStatList["[Buildings] ([Growth])"]!!.equals(Stats(food=2f)))
+    }
+
+    @Test
+    fun carryOverFoodTest() {
+        val civInfo = game.addCiv("[50]% Food is carried over after population increases [in all cities]")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val cityInfo = game.addCity(civInfo, tile, true)
+
+        val foodNecessary = cityInfo.population.getFoodToNextPopulation()
+        cityInfo.population.nextTurn(foodNecessary)
+
+        Assert.assertTrue(cityInfo.population.foodStored == (foodNecessary * 0.5f).toInt())
+    }
+
+
+    @Test
+    fun foodConsumptionBySpecialistsTest() {
+        val civInfo = game.addCiv("[-50]% Food consumption by specialists [in all cities]")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val cityInfo = game.addCity(civInfo, tile, true, initialPopulation = 1)
+
+        val building = game.createBuilding()
+        val specialistName = game.createSpecialist()
+        building.specialistSlots.add(specialistName, 1)
+        cityInfo.population.specialistAllocations[specialistName] = 1
+
+        cityInfo.cityStats.update()
+        print(cityInfo.cityStats.finalStatList)
+        Assert.assertTrue(cityInfo.cityStats.finalStatList["Population"]!!.food == -1f)
+    }
+
+    // endregion growth
+
+    // region happiness
+
+    @Test
+    fun unhappinessFromCitiesPercentageTest() {
+        val civInfo = game.addCiv("[+100]% unhappiness from the number of cities")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val cityInfo = game.addCity(civInfo, tile, true)
+
+        cityInfo.cityStats.update()
+        // Because of some weird design choices, -3.6 is correct, though I would expect it to be -6 instead.
+        // As I'm not certain enough in my feeling that it should be -6, I'm changing the test to fit the code instead of the other way around.
+        // I've also written some text about this in CityStats.updateCityHappiness(). ~xlenstra
+        println(cityInfo.cityStats.happinessList)
+        Assert.assertTrue(abs(cityInfo.cityStats.happinessList["Cities"]!! - -3.6f) < epsilon) // Float rounding errors
+    }
+
+    @Test
+    fun unhappinessFromPopulationTypePercentageChangeTest() {
+        val civInfo = game.addCiv("[-50]% Unhappiness from [Population] [in all cities]")
+        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val cityInfo = game.addCity(civInfo, tile, true, initialPopulation = 4)
+
+        cityInfo.cityStats.update()
+        // This test suffers from the same problems as `unhappinessFromCitiesPercentageTest()`.
+        // This should be -2, I believe, as every pop should add -1 happiness and 4 pop with -50% unhappiness = -2 unhappiness.
+        println(cityInfo.cityStats.happinessList)
+        Assert.assertTrue(abs(cityInfo.cityStats.happinessList["Population"]!! - -1.2f) < epsilon)
+
+        val building = game.createBuilding("[-50]% Unhappiness from [Specialists] [in all cities]")
+        val specialist = game.createSpecialist()
+        building.specialistSlots[specialist] = 2
+        cityInfo.population.specialistAllocations[specialist] = 2
+        cityInfo.cityConstructions.addBuilding(building.name)
+
+        cityInfo.cityStats.update()
+        println(cityInfo.cityStats.happinessList)
+        // This test suffers from the same problems as above. It should be -1, I believe.
+        Assert.assertTrue(abs(cityInfo.cityStats.happinessList["Population"]!! - -0.6f) < epsilon)
+    }
+
+
+    // endregion happiness
+
+
+    // region Great Persons
     @Test
     fun statsSpendingGreatPeople() {
         val civInfo = game.addCiv()
@@ -538,27 +631,8 @@ class GlobalUniquesTests {
         unit.consume()
         Assert.assertTrue(civInfo.gold == 250)
     }
+    // endregion
 
-    @Test
-    fun pillageYieldTest() {
-        game.makeHexagonalMap(2)
-        val civInfo = game.addCiv()
-
-        val tile = game.setTileFeatures(Vector2(0f, 0f), Constants.grassland)
-        val cityTile = game.setTileFeatures(Vector2(2f,0f), Constants.grassland)
-        val cityInfo = game.addCity(civInfo, cityTile, true)
-        cityInfo.population.foodStored = 0 // just to be sure
-        civInfo.addGold(-civInfo.gold) // reset gold just to be sure
-
-        val testImprovement = game.createTileImprovement("Pillaging this improvement yields [+20 Gold, +11 Food]")
-        tile.changeImprovement(testImprovement.name)
-        val unit = game.addUnit("Warrior", civInfo, tile)
-        unit.currentMovement = 2f
-
-        val pillageAction = UnitActionsPillage.getPillageAction(unit)
-        pillageAction?.action?.invoke()
-        Assert.assertTrue("Pillaging should transfer gold to the civ", civInfo.gold == 20)
-        Assert.assertTrue("Pillaging should transfer food to the nearest city", cityInfo.population.foodStored == 11)
-    }
+    // endregion
 
 }
